@@ -27,6 +27,7 @@ interface where possible.
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import sys
+import os
 from osgeo import gdal
 import numpy
 from . import rioserrors
@@ -42,6 +43,9 @@ if sys.version_info[0] > 2:
     # hack for Python 3 which uses str instead of basestring
     # we just use basestring
     basestring = str
+    
+# For supporting the automatic color table generation thing which Sam loves. 
+DEFAULT_AUTOCOLORTABLETYPE = os.getenv('RIOS_DFLT_AUTOCOLORTABLETYPE', default=None)
 
 def isColorColFromUsage(usage):
     "Tells if usage is one of the color column types"
@@ -224,12 +228,19 @@ def writeColumnToBand(gdalBand, colName, sequence, colType=None,
     if hasattr(gdal.RasterAttributeTable, "WriteArray"):
         # new behaviour
         attrTbl = gdalBand.GetDefaultRAT()
-        isFileRAT = True
-
-        # but if it doesn't support dynamic writing
-        # we still ahve to call SetDefaultRAT
-        if not attrTbl.ChangesAreWrittenToFile():
+        if attrTbl is None:
+            # some formats eg ENVI return None
+            # here so we need to be able to cope
+            attrTbl = gdal.RasterAttributeTable()
             isFileRAT = False
+        else:
+
+            isFileRAT = True
+
+            # but if it doesn't support dynamic writing
+            # we still ahve to call SetDefaultRAT
+            if not attrTbl.ChangesAreWrittenToFile():
+                isFileRAT = False
 
     else:
         # old behaviour
@@ -310,8 +321,18 @@ def writeColumnToBand(gdalBand, colName, sequence, colType=None,
                 attrTbl.SetValueAsString(rowNum, colNum, val)
 
     if not isFileRAT:
-        # assume existing bands re-written    
-        gdalBand.SetDefaultRAT(attrTbl)
+        # assume existing bands re-written
+        # Use GDAL's exceptions to trap the error message which arises when 
+        # writing to a format which does not support it
+        usingExceptions = gdal.GetUseExceptions()
+        gdal.UseExceptions()
+        try:
+            gdalBand.SetDefaultRAT(attrTbl)
+        except Exception:
+            pass
+        if not usingExceptions:
+            gdal.DontUseExceptions()
+
 
 def writeColumn(imgFile, colName, sequence, colType=None, bandNumber=1, 
         colUsage=gdal.GFU_Generic):
@@ -433,15 +454,13 @@ def setColorTable(imgfile, colorTblArray, layernum=1):
     (numRows, numCols) = arrayShape
     # Handle the backwards-compatible case of a 4-column array
     if numCols == 4:
-        numCols = 5
-        arrayShape = (numRows, numCols)
-        colorTbl5cols = numpy.zeros(arrayShape, dtype=numpy.uint8)
-        colorTbl5cols[:, 0] = numpy.arange(numRows)
-        colorTbl5cols[:, 1:] = colorTblArray
-        colorTblArray = colorTbl5cols
-        
-    if numCols != 5:
-        raise rioserrors.ArrayShapeError("Color table array has %d columns, expecting 5"%numCols)
+        pixVals = numpy.arange(numRows)
+        colorTbl4cols = colorTblArray
+    elif numCols == 5:
+        pixVals = colorTblArray[:, 0]
+        colorTbl4cols = colorTblArray[:, 1:]
+    else:
+        raise rioserrors.ArrayShapeError("Color table array has %d columns, expecting 4 or 5"%numCols)
     
     # Open the image file and get the band object
     if isinstance(imgfile, gdal.Dataset):
@@ -452,7 +471,7 @@ def setColorTable(imgfile, colorTblArray, layernum=1):
     bandobj = ds.GetRasterBand(layernum)
     
     clrTbl = gdal.ColorTable()
-    maxPixVal = colorTblArray[:, 0].max()
+    maxPixVal = pixVals.max()
     i = 0
     # This loop sets an entry for every pixel value up to the largest given. Imagine
     # bitches if we don't do this. 
@@ -463,12 +482,12 @@ def setColorTable(imgfile, colorTblArray, layernum=1):
         tblMaxVal = 255
         
     for pixVal in range(tblMaxVal+1):
-        while  i < numRows and colorTblArray[i, 0] < pixVal:
+        while  i < numRows and pixVals[i] < pixVal:
             i += 1
         if i < numRows:
-            tblPixVal = colorTblArray[i, 0]
+            tblPixVal = pixVals[i]
             if tblPixVal == pixVal:
-                colEntry = tuple(colorTblArray[i, 1:])
+                colEntry = tuple(colorTbl4cols[i])
             else:
                 colEntry = (0, 0, 0, 0)
         else:
@@ -477,3 +496,90 @@ def setColorTable(imgfile, colorTblArray, layernum=1):
     
     bandobj.SetRasterColorTable(clrTbl)
 
+
+def genColorTable(numEntries, colortype):
+    """
+    Generate a colour table array. The type of colour table generated
+    is controlled by the colortype string. Possible values are:
+    
+        * "rainbow"
+        * "gray"
+        * "random"
+    
+    See corresponding genColorTable_<colortype> function for details of each. 
+    
+    """
+    if colortype == "rainbow":
+        clrTbl = genColorTable_rainbow(numEntries)
+    elif colortype == "gray":
+        clrTbl = genColorTable_gray(numEntries)
+    elif colortype == "random":
+        clrTbl = genColorTable_random(numEntries)
+    else:
+        raise rioserrors.ColorTableGenerationError("Unknown colortype '{}'".format(colortype))
+    return clrTbl
+
+
+def genColorTable_random(numEntries):
+    """
+    Generate a color table array with the given number of entries by assigning 
+    random red/green/blue values. No attempt is made to always generate
+    unique colours, i.e. it is randomly possibly for different entries to
+    have the same colour. 
+
+    Returns the array as a 4 column array, suitable for use with the 
+    :func:`rios.rat.setColorTable` function. 
+    
+    """
+    colorTable = numpy.zeros((numEntries, 4), dtype=numpy.uint8)
+    # RGB entries, random numbers. 
+    colorTable[:, :3] = numpy.random.random_integers(0, 255, size=(numEntries, 3))
+    # Opacity
+    colorTable[:, 3] = 255
+    
+    return colorTable
+
+
+def genColorTable_rainbow(numEntries):
+    """
+    Generate a color table array with the given number of entries, with colors notionally
+    describing a rainbow (i.e. red-orange-yellow-green-blue-indigo-violet). Probably
+    not what a painter would call a rainbow, but it will do. 
+    
+    Returns the array as a 4 column array, suitable for use with the 
+    :func:`rios.rat.setColorTable` function. 
+
+    """
+    colorTable = numpy.zeros((numEntries, 4), dtype=numpy.uint32)
+    # RGB entries. Start at red, blend through to green, then through to blue, in linear steps
+    midNdx = numEntries // 2
+    # Red to green
+    colorTable[0:midNdx, 0] = numpy.mgrid[255:0:midNdx*1j].astype(numpy.uint8)
+    colorTable[0:midNdx, 1] = numpy.mgrid[0:255:midNdx*1j].astype(numpy.uint8)
+    # Green to blue
+    colorTable[midNdx:, 1] = numpy.mgrid[255:0:(numEntries-midNdx)*1j].astype(numpy.uint8)
+    colorTable[midNdx:, 2] = numpy.mgrid[0:255:(numEntries-midNdx)*1j].astype(numpy.uint8)
+    # Opacity
+    colorTable[:, 3] = 255
+    
+    return colorTable
+
+
+def genColorTable_gray(numEntries):
+    """
+    Generate a color table array with the given number of entries, with all 
+    colors being shades of grey. First entry is black, last entry is white. 
+    
+    Returns the array as a 4 column array, suitable for use with the 
+    :func:`rios.rat.setColorTable` function. 
+
+    """
+    colorTable = numpy.zeros((numEntries, 4), dtype=numpy.uint8)
+    # RGB entries. Start at black, blend through to white, in linear steps
+    colorTable[:, 0] = numpy.mgrid[0:255:numEntries*1j].astype(numpy.uint8)
+    colorTable[:, 1] = colorTable[:, 0]
+    colorTable[:, 2] = colorTable[:, 0]
+    # Opacity
+    colorTable[:, 3] = 255
+    
+    return colorTable
